<?php

namespace App\Http\Controllers\Users;

use Exception;
use Throwable;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Mail;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Users\Jobs;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use App\Models\Users\User;
use App\Models\Users\Oauth;
use App\Models\Roles\Role;
use App\Notifications\CreateNewUserNotification;
use App\Notifications\UpdateUserNotification;
use App\Notifications\UpdatePasswordNotification;
use App\Notifications\Users\Notification;
use App\Rules\AllowedEmailsRule;
use App\Jobs\SendMailJob;
use Carbon\Carbon;
use Ramsey\Uuid\Uuid;
use Validator;


class UserController extends Controller 
{
    protected $users;
    protected $oauth_user;

    public function __construct(User $users, Oauth $oauth_user) {
        $this->users = $users;
        $this->oauth_user = $oauth_user;
         //has_scope('users.read');
    }

    public function index(Request $request) 
    {
        try 
        { 
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
    
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";

            if ($request->filled('order_by_column_name')) {

               if (in_array($request->order_by_column_name, $this->getColumnName())) {

                   $order_by_column_name = $request->order_by_column_name;
               }
           }

            $users = User::with(['userRole'])
                    ->where(function ($q) use ($request) {

                        // Global searching...
                        if (isset($request->search) && !empty($request->search) && $request->search != "") {
                            if (!empty($this->getColumnName())) {
                                foreach ($this->getColumnName() as $key => $col) {
                                    if ($key == 0) {
                                        $q->where($col, 'like', '%' . $request->search . '%');
                                    } else {
                                        $q->orWhere($col, 'like', '%' . $request->search . '%');
                                    }
                                }
                            }
                        }

                        // filters
                        if($request->filled('name')) {
                            $q->where('first_name', 'like', '%'.$request->name.'%');
                            $q->orwhere("last_name",'like', '%'.$request->name.'%');
                            $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%' . $request->name . '%');
                        }
                        if($request->filled('email')) {
                            $q->where('email', 'like', '%'.$request->email.'%');
                        }
                    })
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            if(empty($users) ) {
                throw new Exception(config('service.USER_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.USER_LISTING'), 'records' => $users]);
        } 
        catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function store(Request $request) 
    {
         $message = array(
            'password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX'),
            'email.unique'   => config('service.USER_API.MESSAGE.EMAIL_ALREADY_EXISTS'),
            'phone.unique'   => config('service.USER_API.MESSAGE.PHONE_ALREADY_EXISTS'),
            'email.email'   => config('service.USER_API.MESSAGE.VALID_EMAIL')
        );
        $this->validate($request, [
            'first_name' => 'required|max:190',
            'last_name' => 'required|max:190',
            'email' => ['required', 'string', 'email', 'max:255','unique:oauth_users', new AllowedEmailsRule],
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:oauth_users,mobile',
            'role'  => 'required',
            'password' => [
                'required',
                'min:8', // must be at least 8 characters in length
                'max:30',
                'regex:/[a-z]/', // must contain at least one lowercase letter
                'regex:/[A-Z]/', // must contain at least one uppercase letter
                'regex:/[0-9]/', // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'confirm_password' => 'required|same:password'], $message);

        DB::beginTransaction();

        try {
            $input = $request->all();
            $input['password'] = $this->hashPassword($request->password);

            $input['username'] = $request->first_name." ".$request->last_name;
            $insert = $input;
            $insert['role_id'] = $input['role'];
            $insert['created_at']=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));

            unset($insert['password']);
            unset($insert['role']);
            
            if ($user = $this->users->create($insert)) {
                $roles = Role::select('scope')->where('id',$input['role'])->first();
                $oauth = new $this->oauth_user;
                $oauth->user_fk = $user->id;
                $oauth->role_fk = $input['role'];
                $oauth->username = $request->email;
                $oauth->email = $request->email;
                $oauth->mobile = $request->phone;
                $oauth->password = $input['password'];
                $oauth->scope    = $roles->scope;
                $oauth->created_at = convertcurrenttimetoUtc(date('Y-m-d H:i:s'));

                $oauth->save();
                DB::commit();
                
                $greeting = 'Hello ' . $request->first_name . '!';
                $body     = 'You are added as sub-admin on the preventive measures admin with the following roles.'." ".$roles->scope;
                $type     =  'Account Created';

                $details = [
                    'greeting' => $greeting,
                    'body' => $body,
                    'type' => $type,
                    'notification_message' => 'User has been created successfully..',
                    'thanks' => 'Regards,
                    Thanks Preventive Measure',
                ];

                $sendMail = new SendMailJob($request->email, $greeting, $body, $type);
                dispatch($sendMail);

                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_REGISTER'), 'records' => $user]);
            }
            
            throw new Exception(config('service.USER_API.MESSAGE.SOMETHING_WENT_WRONG'));
        } 
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            // This should work as well
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function show($id) 
    {
        try {
            $users = $this->users::with('oauth:user_fk,role_fk as role,username,is_block')->findOrFail($id);
            if (empty($users)) {
                throw new Exception(config('service.USER_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_DETAILS'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function usersDetail($id)
    {
        try {
            $users = User::with(['userRole'])->where('id',$id)->first();
            if (empty($users)) {
                throw new Exception(config('service.USER_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_DETAILS'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function validateInputs(Request $request, $id = null) 
    {
        $message = array(
            'password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX'),
            'email.unique'   => config('service.USER_API.MESSAGE.EMAIL_ALREADY_EXISTS'),
            'phone.unique'   => config('service.USER_API.MESSAGE.PHONE_ALREADY_EXISTS'),
            'email.email'   => config('service.USER_API.MESSAGE.VALID_EMAIL')
        );

        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required|email|max:255|unique:oauth_users',
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:8|max:14|unique:oauth_users,mobile',
            'password' => [
                'required',
                'min:8', // must be at least 8 characters in length
                'regex:/[a-z]/', // must contain at least one lowercase letter
                'regex:/[A-Z]/', // must contain at least one uppercase letter
                'regex:/[0-9]/', // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'confirm_password' => 'required|same:password',
        ];

        if ($id > 0) {
            $uid = Oauth::select('user_id')->where('user_id',$id)->first();

            unset($rules['password'], $rules['confirm_password']);

            //$rules['username'] = 'required|unique:oauth_users,username,'.$uid->user_id.',user_id';
            $rules['email']    = 'email|unique:oauth_users,email,'.$uid->user_id.',user_id';
            $rules['phone']    = 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:8|max:14|unique:oauth_users,mobile,'.$uid->user_id.',user_id';
        }

        return $this->validate($request, $rules, $message);
    }


    public function update(Request $request, $id) 
    {
        $message = array(
            'email.unique'   => config('service.USER_API.MESSAGE.EMAIL_ALREADY_EXISTS'),
            'phone.unique'   => config('service.USER_API.MESSAGE.PHONE_ALREADY_EXISTS'),
            'email.email'   => config('service.USER_API.MESSAGE.VALID_EMAIL'),
            'email.ends_with'   => "We appreciate your interest in joining, however at the moment we only offer this service to those with preventivemeasuresinc.com email addresses."
        );
        $this->validate($request, [
            'first_name' => 'required|max:190',
            'last_name' => 'required|max:190',
            //'email' => 'required|email|unique:admins,email,' . $id,
            // 'email' => 'required|ends_with:yopmail.com,preventivemeasuresinc.com|unique:admins,email,' . $id,
            'email' => 'required|unique:admins,email,' . $id,
            //'email' => ['required', 'string', 'email', 'max:255', new AllowedEmailsRule],
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:admins,phone,' . $id,
            'role'  => 'required',
                ], $message);
        try {

            DB::beginTransaction();
            
            $input = $request->all();
            $user = User::findOrFail($id);
            $adminUser = $this->oauth_user::where('user_fk',$id)->first();

            $user->username = $request->first_name." ".$request->last_name;
            $user->first_name = $request->first_name;
            $user->last_name = $request->last_name;
            $user->email = $request->email;
            $user->phone = $request->phone;
            $user->role_id = $request->role;
            if (!$user->save()) {
                throw new Exception(config('service.USER_API.MESSAGE.SOMETHING_WENT_WRONG'));
            }

            $data = [
                'role_fk' => $input['role'],
                'email' => $request->email,
                'username' => $request->email,
                'mobile' => $request->phone,
                ];  

            Oauth::where('user_fk',$id)->update($data);
            DB::commit();

            $greeting = 'Hi ' . $user->first_name . '!';
            $body     = 'Congrats! Your profile information has been successfully updated.';
            $type     =  'Profile Update';


            $sendMail = new SendMailJob($user->email, $greeting, $body, $type);
            dispatch($sendMail);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Hurray! You have successfully updated your profile.', 'records' => $user]);
        } 
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function updateProfile(Request $request, $id)
    {
         $users = $this->users::findOrFail($id);

         $input = $request->all();

         if ($request->has('image')) {
                //image upload
                $imageName = rand(10000, 9999999) . '-' . time() . '.' . request()->image->getClientOriginalExtension();          
                request()->image->move(storage_path('userProfile'),$imageName);
                $data['image'] = $imageName; 
            }

         $data['first_name'] = $request->first_name;
         $data['last_name']  = $request->last_name;
         $data['phone']      = $request->phone;
         $user = User::where('id',$id)->update($data);  

         $check_exists = $this->oauth_user::where(['user_fk'=>$id])->first();
         if (!empty($check_exists)){
            $update = [];
            $update['mobile']  = $request->phone;

            $result = $this->oauth_user::where('user_fk', $id)->update($update);
        } 

         $greeting = 'Hi ' . $users->first_name . '!';
         $body     = 'Congrats! Your profile information has been successfully updated.';
         $type     =  'Profile Updated';

         $sendMail = new SendMailJob($users->email, $greeting, $body, $type);
         dispatch($sendMail);  

         return response()->json(['status' => 'success', 'message' =>'profile update', 'records' => $user]);

    }


    public function activeInactive(Request $request, $id) {
       
        $this->validate($request,[
            'status' => 'required',
        ]);

        try {
            $save_data['status'] = $request->status;
            $result = $this->users::where('id', $id)->update($save_data);

            if($request->status=="0")
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_STATUS_ACTIVE'),'action'=>$request->status]);
            }
            else
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_STATUS_INACTIVE'),'action'=>$request->status]);
            }
    
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_STATUS')]);

        } 
        catch (Exception $e) { 
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        } 
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }


    public function delete(Request $request, $id) {
    

        try {
            DB::beginTransaction();
            $users = $this->users::find($id);
            $email = $users->email;
            $users->delete();
            $adminUser = $this->oauth_user::where(['user_fk' => $id,'email'=>$email])->delete();
            DB::commit();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_DELETED'), 'records' => $users]);
        } 
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function uploadAvatar(Request $request, $id)
    {

        $this->validate($request, [
           'image' => 'required|mimes:jpeg,png,jpg,svg,gif',
        ]);
        
        try {
            if ($request->has('image')) {
                    $imageName = rand(10000, 9999999) . '-' . time() . '.' . request()->image->getClientOriginalExtension();
                    request()->image->move(storage_path('userProfile'), $imageName);
                    $save_data['image'] = $imageName;
            }
            $result = User::where('id', $id)->update($save_data);
           
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => "User profile picture uploaded successfully!", 'records' => $result]);
        } 
        catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    private function notifyAccountBlock()
    {
        $this->notifyEmail();
        $this->notifySMS();
    }
    
    
    private function validateResetPassword(Request $req, $id)
    {
        $message = array(
            'password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX')
        );
        
        $this->validate($req, [
            'old_password' => 'required',
            'new_password' => [
                'required',
                'min:8', // must be at least 8 characters in length
                'regex:/[a-z]/', // must contain at least one lowercase letter
                'regex:/[A-Z]/', // must contain at least one uppercase letter
                'regex:/[0-9]/', // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'confirm_password' => 'required|same:new_password',
                ], $message);
    }


    public function resetPassword(Request $req, $id) 
    {        

        $old_password = $req->input('old_password');
        $new_password = $req->input('new_password');

        $old_password_encrypted = $this->hashPassword($old_password);

        $user_detail = Oauth::where('password', $old_password_encrypted)
                              ->where('user_fk', $id)->first();

        if (!isset($user_detail)) { 
            $error = ["old_password" => ['The specified password does not match']];
            return response()->json($error, 422);
        }
        
        $this->validateResetPassword($req, $id);
        
        try {
                        
            $user = User::where('id', $id)->first();
            $save_data['password'] = $this->hashPassword($new_password);
            $result = Oauth::where('role_fk', 1)->where('user_fk', $id) 
                            ->update($save_data);
            $msg = 'New Password updated successfully!';

            $greeting = 'Hi ' . $user->first_name . '!';
            $body     = 'New Password updated successfully. your login password is : '.$new_password;
            $type     =  'Password Updated';

            $details = [
                'greeting' => $greeting,
                'body' => $body,
                'type' => $type,
                'notification_message' => 'New Password updated successfully!',
                'thanks' => 'Regards, Thanks',
            ];
            
            /*$role = '1';
            $user->notify(new Notification($details));
            $upd['role'] = $role;
            $notif_id = $user->notifications[0]->id;
            $user->notifications()->find($notif_id)->update($upd);*/

            $sendMail = new SendMailJob($user->email, $greeting, $body, $type);
            dispatch($sendMail);
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => $msg, 'records' => $result]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }

    public function userBlock(Request $req, $id) 
    {
      
        try {

            $action = $req->input('action');

            $save_data['is_block'] = $action;
            $result = DB::table('oauth_users')
                    ->where('user_fk', $id)  // find your user by their id
                    ->where('role_fk', 1)  // find your user by their id
                    ->update($save_data);
            if ($action == 1) {
                
                $msg = 'User Block successfully!';
                $this->notifyAccountBlock();
            } 
            else {
                $msg = 'User Un-block successfully!';
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => $msg, 'records' => $result]);
        } 
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function userAdminCount() 
    {
        try {
            $count = DB::table('admins')->count();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_TOTAL'), 'count' => $count]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function isEmailAvailable(Request $request) 
    {
        $this->validate($request, [
            'email' => 'required|email'
        ]);

        try {
            $email = $request->email;
            $users = $this->oauth_user::where('email', '=', $email)->exists();
            if ($users) {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.EMAIL_ALREADY_EXISTS'), 'records' => $users]);
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.EMAIL_NOT_EXISTS'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function hashPassword($password) 
    {
        return sha1($password);
    }

     public function getScopes($id) {
        
        try {

            $result = DB::table('oauth_users')
                    ->select(array('user_id', 'scope'))
                    ->where('user_fk', $id)
                    ->where('role_fk', 1)->first();
            
            if (isset($result->scope) && trim($result->scope) != '') {
                
                $result->scope = isset($result->scope) ? explode(' ', $result->scope) : [];
            }
             
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Scopes List', 'result' => $result]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }

    public function setScopes(Request $req, $id) {
        
        try {

            $input_data = $req->input('scopes');
            $save_data['scope'] = (isset($input_data) && count($input_data)) ? implode(' ', $input_data) : null;
            $result = Oauth::where('user_fk', $id)
                    ->update($save_data);
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Scopes List', 'result' => $result]);            
        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }

    public function viewAvatar($id)
    {
        try {
            
            $users = $this->users::select('image')->findOrFail($id);
            
            if (empty($users->image)) {
                
                throw new Exception(config('service.USER_API.MESSAGE.USER_NOT_FOUND'));
            }
            
            return $users->image;
                       
        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    } 

    private function getColumnName()
    {
        $address = User::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }

        return ['id'];
    } 
    
}
