<?php

namespace App\Http\Controllers\Vendors;

use Exception;
use Throwable;
use Validator;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Crypt;

use App\Models\Vendors\Vendor;
use App\Models\Vendors\Oauth;
use App\Models\Users\User;
use App\Models\Products\Category;
use App\Models\Products\Product;
use App\Notifications\Vendors\Notification;

use App\Jobs\SendMailJob;
use App\Jobs\VendorMailJob;

class VendorController extends Controller {

    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $users;
    protected $oauth_user;
    
    protected $user_id;
    protected $role_fk;

    public function __construct(Vendor $users, Oauth $oauth_user, Request $request)
    {

        $this->users = $users;
        $this->oauth_user = $oauth_user;
        
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->user_fk = $request->user_fk;
        
        //has_scope('vendors.read');
    }

    
    /**
     * @ user list List api using pagination.
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;

            if ($limit > 500) {
                $limit = 25;
            }

            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }

            $order_by_column_name = "id";

            if (isset($request->order_by_column_name)) {

                if (in_array($request->order_by_column_name, $this->getColumnName())) {

                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $users = $this->users::where('status', '=', 1)
                    ->where(function ($q) use ($request) {

                        // Global searching...
                        if (isset($request->keyword) && !empty($request->keyword) && $request->keyword != "") {
                            if (!empty($this->getColumnName())) {
                                foreach ($this->getColumnName() as $key => $col) {
                                    if ($key == 0) {
                                        $q->where($col, 'like', '%' . $request->keyword . '%');
                                    } else {
                                        $q->orWhere($col, 'like', '%' . $request->keyword . '%');
                                    }
                                }
                            }
                        }
                    })
                    ->where(function ($q) use ($request) {
                        // Column wise searching filter ...
                        $allowableColumn = $this->getColumnName();
                        if (isset($request->column_name) && !empty($request->column_name) && $request->column_value == "null" && in_array($request->column_name, $allowableColumn)) {
                            $q->whereNull($request->column_name);
                        } else if (isset($request->column_name) && !empty($request->column_name) && $request->column_name != "" && in_array($request->column_name, $allowableColumn)) {
                            $q->where($request->column_name, 'like', '%' . $request->column_value . '%');
                        }
                    })
                    ->with('oauth:user_fk,role_fk as role,username,is_block')
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            if (empty($users)) {
                throw new Exception(config('service.USER_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => "List of vendors.", 'records' => $users]);
        } 
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    /**
     * @ service List api using pagination.
     * @return \Illuminate\Http\Response
     */
    public function showMyInfo() 
    {
        try {
            $users = [];
            if (strtolower($this->role_fk) == 'vendor') {
                $users = $this->users::with(['address' => function ($q) {
                                $q->where('user_type', '=', $this->role_fk);
                            }])->find($this->user_id);
            }

            if (empty($users)) {
                throw new Exception(config('service.USER_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_DETAILS'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    
    private function validateInputs(Request $request, $id = null) 
    {
        $message = array(
            'password.regex' => config('service.VENDOR_API.MESSAGE.PASSWORD_REGEX'),
            'email.unique'   => config('service.VENDOR_API.MESSAGE.EMAIL_ALLREADY_EXISTS'),
            'email.email'    => config('service.VENDOR_API.MESSAGE.VALID_EMAIL'),
            'phone.unique'   => config('service.VENDOR_API.MESSAGE.PHONE_ALREADY_EXISTS'),
            'phone.regex'    => config('service.VENDOR_API.MESSAGE.PHONE_REGEX'),
            'phone.min'    => config('service.VENDOR_API.MESSAGE.PHONE_MIN'),
            'phone.max'    => config('service.VENDOR_API.MESSAGE.PHONE_MAX'),
        );

        $rules = [
            'username' => 'required|max:50|unique:oauth_users',
            'first_name' => 'required',
            'last_name' => 'required',
            'language'  => 'in:en,es',
            //'categories' => 'required',
            'shop_no' => 'required',
            'phone_code' => 'required',
            'email' => 'required|email|max:255|unique:oauth_users',
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:8|max:14|unique:oauth_users,mobile',
            'password' => [
                'required',
                'min:8', // must be at least 8 characters in length
                'regex:/[a-z]/', // must contain at least one lowercase letter
                'regex:/[A-Z]/', // must contain at least one uppercase letter
                'regex:/[0-9]/', // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'confirm_password' => 'required|same:password',
        ];

        if ($id > 0) {
            $vid = Oauth::select('user_id')->where('user_fk',$id)->where('role_fk',2)->first();
          
            unset($rules['password'], $rules['confirm_password']);

            $rules['username'] = 'required|unique:oauth_users,username,'.$vid->user_id.',user_id';
            $rules['email'] = 'email|unique:oauth_users,email,'.$vid->user_id.',user_id';
            $rules['phone'] = 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:8|max:14|unique:oauth_users,mobile,'.$vid->user_id.',user_id';
        }

        return $this->validate($request, $rules, $message);
    }
    
    /**
     * @ Register user api.
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {

        //has_scope('vendors.write');

        $this->validateInputs($request);

        try {
            DB::beginTransaction();
            $input = $request->all();
            $input['password'] = $this->hashPassword($request->password);
            
            if ($user = $this->users->create($input)) {
                $roles = DB::table('roles')->select('scope')->where('id',2)->first();
                $oauth = new $this->oauth_user;
                $oauth->user_fk  = $user->id;
                $oauth->role_fk  = 2; // Vendors
                $oauth->username = $request->username;
                $oauth->email    = $request->email;
                $oauth->is_block = 1;
                $oauth->mobile   = $request->phone;
                $oauth->scope    = $roles->scope;
                $oauth->password = $input['password'];
                $oauth->save();
               
                

                $greeting = 'Dear ' . $request->first_name . ',';
                $body     = 'Your vendor acccount is created. Your email : ' . $request->email . ' and your login password is : ' . $request->password . '.';
                $type     =  'Vendor Account Created';

                //vendor mail
                $sendMail = new VendorMailJob($request->email, $greeting, $body, $type);
                dispatch($sendMail);

                //admin mail
                $greetingAdmin = 'Dear Admin';
                $bodyAdmin     = 'A new vendor is recently registered. Please review now.';
                $typeAdmin     = 'New Vendor Registered'; 

                $adminMail = new SendMailJob(env('ADMIN_EMAIL'), $greetingAdmin, $bodyAdmin, $typeAdmin);
                dispatch($adminMail);

                //db notification for admin
                $admin = User::select('id')->where('id','1')->first();

                $details = [
                    'greeting' => $greetingAdmin,
                    'body' => $bodyAdmin,
                    'type' => $typeAdmin,
                    'notification_message' => 'Please click here to check.',
                    'vendor_id' => $user->id,
                    'thanks' => 'Regards, Thanks',
                ];
             
                $role = '1'; // Admin
                $admin->notify(new Notification($details));
                $upd['role'] = $role;
                $notif_id = $admin->notifications[0]->id;
                $admin->notifications()->find($notif_id)->update($upd);

                DB::commit();

                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.VENDOR_REGISTER'), 'records' => $user]);
            }
        } 
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            // This should work as well
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function addDetails(Request $request, $id) 
    {
        has_scope('vendors.write');
        
        $this->validate($request, [
            'address' => 'required',
            'latitude' => 'required',
            'longitude' => 'required'
        ]);
         

        try {

            $vendor = Vendor::find($id);

            if($request->latitude != $vendor->latitude || $request->longitude != $vendor->longitude) {
            
                //db notification for admin
                $admin = User::select('id')->where('id', '1')->first();

                $greetingAdmin = 'Dear Admin,';
                $bodyAdmin     = $vendor->first_name.' has updated their address. Please check now.';
                $typeAdmin     = 'Vendor Updated Address'; 

                $details = [
                        'greeting' => $greetingAdmin,
                        'body' => $bodyAdmin,
                        'type' => $typeAdmin,
                        'notification_message' => 'Please click here to check.',
                        'vendor_id' => $id,
                        'thanks' => 'Regards,
                        Thanks',
                ];

                $role = '1'; // Admin
                $admin->notify(new Notification($details));
                $upd['role'] = $role;
                $notif_id = $admin->notifications[0]->id;
                $admin->notifications()->find($notif_id)->update($upd);
            }
           

            $vendor->address    = $request->address;
            $vendor->latitude   = $request->latitude;
            $vendor->longitude  = $request->longitude;
            
            $vendor->city     = isset($request->city) ? $request->city : '';
            $vendor->country  = isset($request->country) ? $request->country : '';
            $vendor->state    = isset($request->state) ? $request->state : '';
            $vendor->zipcode  = isset($request->zipcode) ? $request->zipcode : '';
            
            $vendor->update();

            
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.DETAILS_ADDED'), 'records' => $vendor]);
        
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
   
    }

    
    /**
     * @ vendor detail api.
     * @return \Illuminate\Http\Response
     */
    public function show($id) 
    {
        try {
            
            $users = $this->users::with('oauth:user_fk,role_fk as role,username,is_block')->where('id', $id)->first();
            
            if (empty($users)) {
                
                throw new Exception(config('service.USER_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => "Vendor information.", 'records' => $users]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    
    /**
     * @ update user api .
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        has_scope('vendors.write');

        $users = $this->users::findOrFail($id);
       
        $this->validateInputs($request, $id);

        try {
         
            if($request->email != $users->email && $request->role_fk == '2') {
                $decodedEmail = Crypt::encryptString($request->email);
                $update = Vendor::where('id', $id)->update(['token' => $decodedEmail]);
                $this->updateEmail($id,$request->email,$decodedEmail);
            }

            $input = $request->all();

            if ($request->has('image')) {
                $imageName = rand(10000, 9999999) . '-' . time() . '.' . request()->image->getClientOriginalExtension();
                request()->image->move(storage_path('vendorProfile'), $imageName);
                $input['image'] = $imageName;
            }
            
            if (!$users->update($input)) {
                throw new Exception(config('service.USER_API.MESSAGE.SOMETHING_WENT_WRONG'));
            }

            $check_exists = $this->oauth_user::where([['role_fk', '=', '2'],['user_fk', '=', $id],])->first();

            if (!empty($check_exists) && $check_exists) {
                $data = array();
                $data['username'] = $request->username;
                $data['email'] = $request->email;
                $data['mobile'] = $request->mobile;
                $this->oauth_user::where([['role_fk', '=', '2'],['user_fk', '=', $id],])->update($data);
            }
                
            
            
            $greeting = 'Hi ' . $request->first_name . '!';
            $body     = 'Your profile information has been successfully updated! Please review now.';
            $type     =  'Vendor Profile Update';

            $details = [
                'greeting' => $greeting,
                'body' => $body,
                'type' => $type,
                'notification_message' => 'You have updated your profile information.',
                'thanks' => 'Regards, Thanks',
            ];
            
            $role = '2';
            $users->notify(new Notification($details));
            $upd['role'] = $role;
            $notif_id = $users->notifications[0]->id;
            $users->notifications()->find($notif_id)->update($upd);

            $sendMail = new SendMailJob($users->email, $greeting, $body, $type);
            dispatch($sendMail);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.PROFILE_UPDATED'), 'records' => $users]);
        } 
        catch (Exception $e) {
            
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    private function getColumnName()
    {
        $address = $this->users::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }

        return ['id'];
    }
    
    
    public function vendorCount() 
    {
        try {
            
            $users = $this->users::count();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => "Total number of vendors.", 'count' => $users]);
        } 
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    public function isEmailAvailable(Request $request)
    {
        $this->validate($request, [
            'email' => 'required|email'
        ]);

        try {
            $email = $request->email;
            $users = $this->oauth_user::where('email', '=', $email)->exists();
            if ($users) {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.EMAIL_ALREADY_EXISTS'), 'exists' => $users]);
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.EMAIL_NOT_EXISTS'), 'exists' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete(Request $request, $id)
    {

        has_scope('vendors.delete');

        $users = $this->users::findOrFail($id);
       

        $pass = "150464c36f0ddff2ce423db7d6851c3c48d577f3";
        try {
            $users = $users->delete();

            $this->oauth_user::where('user_fk', $id)->where('role_fk', 2)->delete();
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.VENDOR_DELETED'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function hashPassword($password) 
    {        
        return sha1($password);
    }

    private function validateResetPassword(Request $req, $id)
    {
        $message = array(
            'new_password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX')
        );
        
        $this->validate($req, [
            'old_password' => 'required',
            'new_password' => [
                'required',
                'min:8', // must be at least 8 characters in length
                'regex:/[a-z]/', // must contain at least one lowercase letter
                'regex:/[A-Z]/', // must contain at least one uppercase letter
                'regex:/[0-9]/', // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'confirm_password' => 'required|same:new_password',
                ], $message);
    }

    private function validateResetPasswordAdmin(Request $req, $id)
    {
        $message = array(
            'new_password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX')
        );
        
        $this->validate($req, [
            //'old_password' => 'required',
            'new_password' => [
                'required',
                'min:8', // must be at least 8 characters in length
                'regex:/[a-z]/', // must contain at least one lowercase letter
                'regex:/[A-Z]/', // must contain at least one uppercase letter
                'regex:/[0-9]/', // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'confirm_password' => 'required|same:new_password',
                ], $message);
    }


    public function resetPassword(Request $request, $id) 
    {        
        has_scope('vendors.write');
        
        $old_password = $request->input('old_password');
       
        $new_password = $request->input('new_password');

        $old_password_encrypted = $this->hashPassword($old_password);

        $user_detail = oauth::where('password', $old_password_encrypted)
                              ->where('role_fk','2')->where('user_fk', $id)->first();
        
        if($request->role_fk == '1') {
             $this->validateResetPasswordAdmin($request, $id);
        } else {
            $this->validateResetPassword($request, $id);

            if (!isset($user_detail)) {
            
                $error = ["old_password" => ['The specified password does not match']];
    
                return response()->json($error, 422);
            }
            
        }
        
        
        try {
                        
            $vendor = Vendor::where('id', $id)->first();

            $save_data['password'] = $this->hashPassword($new_password);
            $result = DB::table('oauth_users')
                        ->where('role_fk',2)
                        ->where('user_fk', $id)  // find your user by their id
                        ->update($save_data);

            $greeting = 'Hi ' . $vendor->first_name . '!';
            $body     = ' Your password has been changed successfully! Please login now.';
            $type     =  'Password Updated';

            $details = [
                'greeting' => $greeting,
                'body' => $body,
                'type' => 'Vendor Password Updated.',
                'notification_message' => 'New Password updated successfully!',
                'thanks' => 'Regards, Thanks',
            ];
            
            $role = '2';
            $vendor->notify(new Notification($details));
            $upd['role'] = $role;
            $notif_id = $vendor->notifications[0]->id;
            $vendor->notifications()->find($notif_id)->update($upd);

            $sendMail = new SendMailJob($vendor->email, $greeting, $body, $type);
            dispatch($sendMail);
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.PASSWORD_UPDATED')]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }

    //update category 
    public function updateCategory(Request $request, $id) {
        
        has_scope('vendors.write');

        $vendor = Vendor::findOrFail($id);

        $this->validate($request, [
           'categories' => 'required',
        ]);
        
        try {
            $data = Vendor::where('id',$id)->update(['categories' => implode(',',$request->categories)]);

            $greeting = 'Hi ' . $vendor->first_name . '!';
            $body     = 'Your categories have been updated successfully!';
            $type     =  'Categories Change';

            $sendMail = new SendMailJob($vendor->email, $greeting, $body, $type);
            dispatch($sendMail);

            //db notification for admin
            $admin = User::select('id')->where('id', '1')->first();

            $greetingAdmin = 'Dear Admin,';
            $bodyAdmin     = $vendor->first_name.' has updated their categories. Please check now.';
            $typeAdmin     = 'Vendor Updated Categories'; 

            $details = [
                    'greeting' => $greetingAdmin,
                    'body' => $bodyAdmin,
                    'type' => $typeAdmin,
                    'notification_message' => 'Please click here to check.',
                    'vendor_id' => $id,
                    'thanks' => 'Regards, Thanks',
            ];

            $role = '1'; // Admin
            $admin->notify(new Notification($details));
            $upd['role'] = $role;
            $notif_id = $admin->notifications[0]->id;
            $admin->notifications()->find($notif_id)->update($upd);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.VENDOR_API.MESSAGE.CATEGORY_UPDATED'), 'records' => $data]);
        } 
        catch (Exceptio $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);

        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }
 
    //upload image
    public function uploadAvatar(Request $request, $id) {
        has_scope('vendors.write');
        
        $this->validate($request, [
          'image' => 'required|mimes:jpeg,png,jpg,svg,gif'
        ]);
        
        try {

            if($request->has('image')) {
                $imageName = rand(10000, 9999999) . '-' . time() . '.' . request()->image->getClientOriginalExtension();
                request()->image->move(storage_path('userProfile'), $imageName);
                $save_data['image'] = $imageName;
            }

            $result = Vendor::where('id',$id)->update($save_data);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.IMAGE_UPDATED'), 'records' => $result]);


        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }

    }

    public function viewAvatar($id)
    {
        try {
            
            $vendor = Vendor::select('image')->find($id);
            
            if (empty($vendor->image)) {
                
                return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.USER_API.MESSAGE.NOT_FOUND')]);
            }
            
            return $vendor->image;
                       
        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    } 


    //approve vendor
    public function approve(Request $request, $id) {
        has_scope('vendors.write');

        $vendor = Vendor::findOrFail($id);

        $this->validate($request, [
           'status' => 'required|in:0,1'
        ]);

        try {

            $status  = $request->status;
            $approve = Vendor::where('id', $id)->update(['approval_status' =>$status]);
            
            if($status == '1') {
                //apporved

                Oauth::where('user_fk',$id)->where('role_fk',2)->update(['is_block' => '0']);
                $body = 'Congratulations! Your account has been approved. Please login now.';
                $type =  'Vendor Account Approved';

                $message =   config('service.VENDOR_API.MESSAGE.VENDOR_APPROVED');
            } else {
                //rejected

                $body = 'Your account has been rejected.';
                $body .= $request->reason;
                $type =  'Vendor Account Rejected';

                $message =   config('service.VENDOR_API.MESSAGE.VENDOR_REJECTED');
            }
                    
            $greeting = 'Dear ' . $vendor->first_name . ',';

            $sendMail = new VendorMailJob($vendor->email, $greeting, $body, $type);
            dispatch($sendMail);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => $message]);

        } 
        catch (Exception $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        } 
        catch (Throwable $e) {
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }

    //edit email
    private function updateEmail($id,$email,$token) {
        has_scope('vendors.write');
    
        $vendor = Vendor::findOrFail($id);
     
        try{

                $data['name'] = $vendor->first_name;

                // Genereate link for verify email
                $data['link'] =  env('FRONT_END') . '/' . 'vendor/verify-email' . '/' . $token;
                
                        
                // Trigger mail
                Mail::send('emails.Vendors.email_verify',['data'=>$data], function($message) use ($email) {
                    $message->to($email);
                    $message->subject('Verify Email');
                });


                return return_json(["status" => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.VENDOR_EMAIL_UPDATE')]);  

        }
        catch(Exception $e)
        {
            return return_json(["error" => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage() , 'records' => []]);
        }
        catch(Throwable $e)
        {
            return return_json(["error" => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage() , 'records' => []]);
        }
        
    }

    // verify emial
    public function VerifyEmail(Request $request, $token) {
        try{

            $email = Crypt::decryptString($token);

            $vid    = Vendor::select('id','first_name')->where('token',$token)->first();

            if(!$vid) {
               return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') ,'message' => config('service.VENDOR_API.MESSAGE.INVALID_LINK') , ]);
            }

           $data  = array();
           $data['email'] = $email;
           $data['token'] = null;
           
           Vendor::where('token',$token)->update($data);
    
           $oauth  = Oauth::where('user_fk',$vid->id)->where('role_fk','2')->update(['email' => $email]);

           //db notification for admin
            $admin = User::select('id')->where('id','1')->first();

            $greetingAdmin = 'Dear Admin,';
            $bodyAdmin     = $vid->first_name.' has changed their email. Please click here to review!.';
            $typeAdmin     = 'Vendor Updated Profile'; 

            $details = [
                    'greeting' => $greetingAdmin,
                    'body' => $bodyAdmin,
                    'type' => $typeAdmin,
                    'notification_message' => 'Please click here to check.',
                    'vendor_id' => $vid->id,
                    'thanks' => 'Regards,Thanks',
            ];

            $role = '1'; // admin
            $admin->notify(new Notification($details));
            $upd['role'] = $role;
            $notif_id = $admin->notifications[0]->id;
            $admin->notifications()->find($notif_id)->update($upd);

           return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS') , 'message' => config('service.VENDOR_API.MESSAGE.VENDOR_EMAIL_UPDATED') ]);

        } catch(Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
        catch(Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
      
    }

    //update number
    public function updateNo(Request $request, $id) {
        has_scope('vendors.write');

        $vendor = Vendor::findOrFail($id);

        $message = array(
            'phone.unique'   => config('service.VENDOR_API.MESSAGE.PHONE_ALREADY_EXISTS'),
            'phone.regex'    => config('service.VENDOR_API.MESSAGE.PHONE_REGEX'),
            'phone.min'    => config('service.VENDOR_API.MESSAGE.PHONE_MIN'),
            'phone.max'    => config('service.VENDOR_API.MESSAGE.PHONE_MAX'),
        );

        $this->validate($request, [
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:8|max:14|unique:oauth_users,mobile,' . $id.',user_fk',
        ],$message);
        try {
            $otp    = random_int(1111, 9999);
            $update = Vendor::where('id',$id)->update(['otp' =>  $otp]);


            $greeting = 'Hi ' . $vendor->first_name . '!';
            $body     = 'Your OTP is '. $otp;
            $type     = 'Mobile No Verify OTP';

            $sendMail = new SendMailJob($vendor->email, $greeting, $body, $type);
            dispatch($sendMail);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS') , 'message' => config('service.VENDOR_API.MESSAGE.OTP_SENT') ]);


        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
    }

    //verify otp 
    public function verifyOtp(Request $request, $id) {
        $message = array(
            'phone.unique'   => config('service.VENDOR_API.MESSAGE.PHONE_ALREADY_EXISTS'),
            'phone.regex'    => config('service.VENDOR_API.MESSAGE.PHONE_REGEX'),
            'phone.min'    => config('service.VENDOR_API.MESSAGE.PHONE_MIN'),
            'phone.max'    => config('service.VENDOR_API.MESSAGE.PHONE_MAX'),
        );
        
        $this->validate($request, [
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:8|max:14',
            'otp'   => 'required|min:4|max:4'
        ],$message);

        try {
            $vendor = Vendor::select('otp','first_name')->where('id',$id)->where('otp',$request->otp)->first();

            if(!$vendor) {
                return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.INVALID_OTP')]);
            }
             
            $data = array();
            $data['phone'] = $request->phone;
            $data['otp']   = null;
            Vendor::where('id',$id)->update($data);

            Oauth::where('user_fk',$id)->where('role_fk','2')->update(['mobile' => $request->phone]);


            //db notification for admin
            $admin = User::select('id')->where('id','1')->first();

            $greetingAdmin = 'Dear Admin,';
            $bodyAdmin     = $vendor->first_name.' has changed their mobile no. Please click here to review!.';
            $typeAdmin     = 'Vendor Updated Profile'; 

            $details = [
                    'greeting' => $greetingAdmin,
                    'body' => $bodyAdmin,
                    'type' => $typeAdmin,
                    'notification_message' => 'Please click here to check.',
                    'vendor_id' => $id,
                    'thanks' => 'Regards,Thanks',
            ];

            $role = '1'; // admin
            $admin->notify(new Notification($details));
            $upd['role'] = $role;
            $notif_id = $admin->notifications[0]->id;
            $admin->notifications()->find($notif_id)->update($upd);
 
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS') , 'message' => config('service.VENDOR_API.MESSAGE.MOBILE_NO_UPDATED') ]);
        }catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }

    }

    public function categoryCount($id) {
        has_scope('vendors.read');

        $vendor = Vendor::findOrFail($id);

        try {
            $categories = $vendor->categories!=null ? explode(",",$vendor->categories) : [];
            $count = count($categories);
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.VENDOR_CATEGORIES'), 'count' => $count]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function changeLanguage(Request $request, $id) {
        has_scope('vendors.write');

        $vendor = Vendor::findOrFail($id);

        $this->validate($request, [
            'language' => 'required|in:en,es',
        ]);

        try {
            $update = Vendor::where('id',$id)->update(['language' =>  $request->language]);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.LANGUAGE_UPDATED'), 'records' => $update]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    } 

    public function categories($id) {
        has_scope('vendors.read');

        $vendor = Vendor::findOrFail($id);

        try {
            $categories  = explode(',', $vendor->categories);
        
            $vendorCategories = Category::whereIn('id',$categories)->whereNull('parent_id')->get();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.VENDOR_ALL_CATEGORIES'), 'records' => $vendorCategories]);
       
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function productCount($id) {
        has_scope('vendors.read');

        $vendor = Vendor::findOrFail($id);

        try {
           $count = Product::where('vendor_id',$id)->count();
           return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.VENDOR_PRODUCTS'), 'records' => $count]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function comission(Request $request, $id) {
        has_scope('vendors.write');

        $this->validate($request, [
            'commission' => 'required|numeric',
            'commission_type' => 'required|in:percent,amount',
        ]);

        $vendor = Vendor::findOrFail($id);
       
        
        try {

            $input = $request->all();
            $save  = $vendor->update($input);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.VENDOR_API.MESSAGE.COMISSION_ADDED'), 'records' => $save]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        
    }

}
