<?php

namespace App\Http\Controllers\v4\Contractors;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Contractors\Contractor;

use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;

use App\Models\Contractors\Jobs;
use App\Models\Contractors\Customer;
use App\Models\Contractors\Oauth;


use App\Notifications\CreateNewUserNotification;

class CustomerController extends Controller {

    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $users;
    
    protected $oauth_user;
    protected $jobs;
    protected $user_id;
    protected $role_fk;

    public function __construct(Customer $users, Oauth $oauth_user, Jobs $jobs, Request $request) {

        $this->users = $users;
        $this->oauth_user = $oauth_user;
        $this->jobs = $jobs;

        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
    }

    /**
     * @ user list List api using pagination.
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request) {

        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;

            if ($limit > 500) {
                $limit = 25;
            }

            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }

            $order_by_column_name = "id";

            if (isset($request->order_by_column_name)) {

                if (in_array($request->order_by_column_name, $this->getColumnName())) {

                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $users = $this->users::where('status', '=', 1)
                    ->where(function ($q) use ($request) {

                        // Global searching...
                        if (isset($request->search) && !empty($request->search) && $request->search != "") {
                            if (!empty($this->getColumnName())) {
                                foreach ($this->getColumnName() as $key => $col) {
                                    if ($key == 0) {
                                        $q->where($col, 'like', '%' . $request->search . '%');
                                    } else {
                                        $q->orWhere($col, 'like', '%' . $request->search . '%');
                                    }
                                }
                            }
                        }
                    })
                    ->where(function ($q) use ($request) {
                        // Column wise searching filter ...
                        $allowableColumn = $this->getColumnName();
                        if (isset($request->column_name) && !empty($request->column_name) && $request->column_name != "" && in_array($request->column_name, $allowableColumn)) {
                            $q->where($request->column_name, 'like', '%' . $request->column_value . '%');
                        }
                    })
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            if (empty($users)) {
                throw new Exception(config('service.USER_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_DETAILS'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ Service List api using pagination.
     * @return \Illuminate\Http\Response
     */
    public function showMyInfo() {
        try {
            $users = [];
            if (strtolower($this->role_fk) == 'customer') {
                $users = $this->users::with(['address' => function ($q) {
                                $q->where('user_type', '=', $this->role_fk);
                            }])->find($this->user_id);
            }

            if (empty($users)) {
                throw new Exception(config('service.USER_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_DETAILS'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ Register user api.
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request) {
        $message = array(
            'password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX')
        );
        $this->validate($request, [
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required|email|max:255|unique:customers',
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:customers',
            'password' => [
                'required',
                'min:8', // must be at least 8 characters in length
                'regex:/[a-z]/', // must contain at least one lowercase letter
                'regex:/[A-Z]/', // must contain at least one uppercase letter
                'regex:/[0-9]/', // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'confirm_password' => 'required|same:password',
                ], $message);

        try {
            DB::beginTransaction();
            $input = $request->all();
            $input['password'] = $this->hashPassword($request->password);
            if ($user = $this->users->create($input)) {
                $oauth = new $this->oauth_user;
                $oauth->user_id = $user->id;
                $oauth->role_fk = 'customer';
                $oauth->username = $request->email;
                $oauth->password = $input['password'];
                $oauth->first_name = $request->first_name;
                $oauth->last_name = $request->last_name;
                $oauth->email = $request->email;
                $oauth->phone = $request->phone;
                $oauth->save();
                DB::commit();

                $details = [
                    'greeting' => 'Hi ' . $request->first_name . '!',
                    'body' => 'Your email : ' . $request->email . ' and your login password is : ' . $request->password,
                    'database_notification_message' => 'New user has been created successfully.',
                    'thanks' => 'Welcome to ZION!!',
                ];
                $role = 'customer';
                $user->notify(new CreateNewUserNotification($role, $details));
                $upd['user_type'] = $role;
                $notif_id = $user->notifications[0]->id;
                $user->notifications()->find($notif_id)->update($upd);

                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_REGISTER'), 'records' => $user]);
            }
            throw new Exception(config('service.USER_API.MESSAGE.SOMETHING_WENT_WRONG'));
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            // This should work as well
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ Service List api using pagination.
     * @return \Illuminate\Http\Response
     */
    public function show($id) {
        try {
            $users = $this->users;
            $users = $users::with(['address' => function ($q) {
                            $q->where('user_type', '=', 'customer');
                        }, 'jobs' => function ($q) {
                            $q->select('user_id',
                                    DB::raw('(CASE WHEN status = 1 THEN COUNT(status) ELSE 0 END) AS confirmed'),
                                    DB::raw('(CASE WHEN status = 2 THEN COUNT(status) ELSE 0 END) AS on_the_way'),
                                    DB::raw('(CASE WHEN status = 3 THEN COUNT(status) ELSE 0 END) AS in_process'),
                                    DB::raw('(CASE WHEN status = 4 THEN COUNT(status) ELSE 0 END) AS completed'),
                                    DB::raw('(CASE WHEN status = 5 THEN COUNT(status) ELSE 0 END) AS cancelled'),
                            )->groupBy(['user_id', 'status']);
                        }])->find($id);

            $store = [];
            if (isset($users->jobs) && count($users->jobs) > 0) {
                foreach ($users->jobs as $key => $val) {
                    if ($val->confirmed > 0) {
                        $store[0]['confirmed_job'] = $val->confirmed;
                    }
                    if ($val->completed > 0) {
                        $store[0]['completed_job'] = $val->completed;
                    }
                    if ($val->cancelled > 0) {
                        $store[0]['cancelled_job'] = $val->cancelled;
                    }
                }
            }

            $users->setRelation('jobs', collect($store));

            if (empty($users)) {
                throw new Exception(config('service.USER_API.MESSAGE.USER_NOT_FOUND'));
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_DETAILS'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ update user api .
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id) {
        $message = array(
            'password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX')
        );
        $this->validate($request, [
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required|email|unique:customers,email,' . $id,
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:customers,phone,' . $id,
            'password' => [
                'min:8', // must be at least 8 characters in length
                'regex:/[a-z]/', // must contain at least one lowercase letter
                'regex:/[A-Z]/', // must contain at least one uppercase letter
                'regex:/[0-9]/', // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'image' => $request->has('image') ? 'required|mimes:jpeg,png,jpg,svg,gif' : "",
                ], $message);

        try {
            DB::beginTransaction();
            $users = $this->users::find($id);
            $input = $request->all();
            if (!empty($input['password'])) {
                $input['password'] = $this->hashPassword($input['password']);
            } else {
                $input = Arr::except($input, ['password']);
            }
            if ($request->has('image')) {
                $image = $request->file('image');
                $imagename = time() . '.' . $image->getClientOriginalName();
                $image->move(public_path('assets/images/users'), $imagename);
                $input['image'] = @$imagename;
            }

            if (!$users->update($input)) {
                throw new Exception(config('service.USER_API.MESSAGE.SOMETHING_WENT_WRONG'));
            }

            $check_exists = $this->oauth_user::where([
                        ['role_fk', '=', 'customer'],
                        ['user_id', '=', $id],
                    ])->first();
            if (!empty($check_exists) && $check_exists) {
                $oauth = $this->oauth_user::find($check_exists->id);
                $oauth->username = $request->email;
                if (!empty($input['password'])) {
                    $oauth->password = $input['password'];
                }
                $oauth->first_name = $request->first_name;
                $oauth->last_name = $request->last_name;
                $oauth->email = $request->email;
                $oauth->phone = $request->phone;
                $oauth->update();
            }
            $users['image_path'] = url('/assets/images/users');
            DB::commit();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_UPDATED'), 'records' => $users]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function customerCount() {
        try {
            $users = $this->users::count();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_TOTAL'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function isEmailAvailable(Request $request) {
        $this->validate($request, [
            'email' => 'required|email'
        ]);

        try {
            $email = $request->email;
            $users = $this->users::where('email', '=', $email)->exists();
            if ($users) {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.EMAIL_ALREADY_EXISTS'), 'records' => $users]);
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.EMAIL_NOT_EXISTS'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete(Request $request, $id) {
        $pass = "150464c36f0ddff2ce423db7d6851c3c48d577f3";
        try {
            $users = $this->users::find($id)->delete();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.USER_DELETED'), 'records' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function allCustomerJobs(Request $request, $id) {
        try {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if (isset($request->order_by_column_name)) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $jobs = $this->jobs::select(['booking.id', 'booking.user_id', 'booking.contractor_id', 'service-category.sub_category_name', 'booking.booking_date', 'booking.booking_time', 'job_tracking.status'])
                    ->join('booking', 'booking.id', '=', 'job_tracking.booking_id')
                    ->join('services', 'services.id', '=', 'booking.service_id')
                    ->join('service-category', 'service-category.id', '=', 'booking.service_cat_id')
                    ->where(function ($q) use ($id) {
                        $q->where('booking.user_id', '=', $id);
                    })
                    ->where(function ($q) use ($request) {
                        // Global searching...
                        if (isset($request->search) && !empty($request->search) && $request->search != "") {
                            if (!empty($this->getJobsColumnName())) {
                                foreach ($this->getJobsColumnName() as $key => $col) {
                                    if ($key == 0) {
                                        $q->where($col, 'like', '%' . $request->search . '%');
                                    } else {
                                        $q->orWhere($col, 'like', '%' . $request->search . '%');
                                    }
                                }
                            }
                        }
                    })
                    ->where(function ($q) use ($request) {
                        // Column wise searching filter ...
                        $allowableColumn = $this->getJobsColumnName();
                        $request->column_name = 'job_tracking.' . $request->column_name;
                        // echo "<pre>"; print_r($request->column_name); die;
                        if (isset($request->column_name) && !empty($request->column_name) && $request->column_name != "" && in_array($request->column_name, $allowableColumn)) {
                            $q->where($request->column_name, 'like', '%' . $request->column_value . '%');
                        } else {
                            $request->column_name = 'booking.' . $request->column_name;
                            if (isset($request->column_name) && !empty($request->column_name) && $request->column_name != "" && in_array($request->column_name, $allowableColumn)) {
                                $q->where($request->column_name, 'like', '%' . $request->column_value . '%');
                            }
                        }
                    })
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.JOBS_LIST'), 'records' => $jobs]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $address = $this->users::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }

        return ['id'];
    }

    private function getJobsColumnName() {
        $col_name = [];
        $col = $this->jobs::first();
        if (!empty($col) && $col != "") {
            // return array_keys($col->toArray());
            foreach (array_keys($col->toArray()) as $key => $value) {
                $col_name[] = 'job_tracking.' . $value;
            }
            $extraCol = ['booking.id', 'booking.user_id', 'booking.contractor_id', 'service-category.sub_category_name', 'booking.booking_date', 'booking.booking_time'];
            return array_merge($col_name, $extraCol);
        }

        return ['id'];
    }

    private function hashPassword($password) {
        return sha1($password);
    }

}
