<?php

namespace App\Models\Customers;

use Illuminate\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Laravel\Lumen\Auth\Authorizable;

use Illuminate\Notifications\Notifiable;
use App\Models\Customers\Address;
use App\Models\Customers\Jobs;
use Illuminate\Database\Eloquent\SoftDeletes;
use Spatie\Activitylog\Traits\LogsActivity;
use Spatie\Activitylog\LogOptions;
use App\Jobs\SendTwoFactorJob;
use Carbon\Carbon;

class Customer extends Model implements AuthenticatableContract, AuthorizableContract
{
    use Authenticatable, Authorizable, HasFactory, Notifiable, SoftDeletes,LogsActivity;

    protected $table = 'customers';
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'first_name',
         'last_name', 
         'birth_date',
         'gender',
         'other_gender',
         'email',
         'password',
         'phone_code',
         'phone', 
         'address',
         'apartment',
         'suite_number',
         'image',
         'other_race',
         'race_id',
         'ethnicity_id',
         'religious_id',
         'relationship_id',
         'register_step',
         'register_status',
         'current_provider_id',
         'email_confirmed',
         'email_confirm_token',
         'language_id',
         'plan_id',
         'appointment_session',
         'stripe_customer_id',
         'stripe_subscription_id',
         'purchased_date',
         'expired_date',
         'start_deactivate_date_time',
         'end_deactivate_date_time',
         'device_id',
         'device_token',
         'device_type',
         'two_factor',
         'two_factor_code',
         'two_factor_expires_at',
         'otp',
         'otp_validity',
         'sms_otp',
         'height_feet',
         'height_inch',
         'login_attempt',
         'is_blocked',
         'login_attempt_time',
         'last_updated_by',
         'last_updated_at',
         'last_viewed_at',
         'latitude',
         'longitude',
         'timezone',
         'pin',
         'city',
         'country',
         'state',
         'zipcode',
         'token',
         'country',
         'city',
         'verified',
         'last_login_at'
    ];

    /**
     * The attributes excluded from the model's JSON form.
     *
     * @var array
     */
    protected $hidden = [
        'password',
    ];

    // protected $casts = [
    //     'created_at' => 'datetime:Y-m-d H:i:s',
    //     'updated_at' => 'datetime:Y-m-d H:i:s',
    // ];

    public function generateTwoFactorCode()
    {
        $digits = 4;
        $toEmail = decryptString($this->email);
        //$toEmail = $this->email;
        $toName = decryptString($this->first_name);
        $otpCode = rand(pow(10, $digits-1), pow(10, $digits)-1);
        $sendMail =  new SendTwoFactorJob($toName,$toEmail,$otpCode); 
        dispatch($sendMail);

        $this->timestamps = false;
        $this->two_factor_code = $otpCode;
        $this->two_factor_expires_at = strtotime(date("Y-m-d H:i:s"));
        $this->save();  

    }

    public function resetTwoFactorCode()
    {
        $this->timestamps = false;
        $this->two_factor_code = null;
        $this->two_factor_expires_at = null;
        $this->save();
    }
    
    public function relationshipStatus(){ 
        return $this->belongsTo('App\Models\Customers\Relationship','relationship_id','id');       
    }

    public function adminUser(){ 
        return $this->belongsTo('App\Models\Users\Admin','last_updated_by','id');       
    }

    public function adminUserView(){ 
        return $this->belongsTo('App\Models\Users\Admin','last_viewed_by','id');       
    }

    public function race(){ 
        return $this->belongsTo('App\Models\Customers\Race','race_id','id');       
    }

    public function states(){ 
        return $this->belongsTo('App\Models\Providers\State','state','id');       
    }

    public function ethnicity(){ 
        return $this->belongsTo('App\Models\Customers\Ethnicity','ethnicity_id','id');       
    }

    public function religious(){ 
        return $this->belongsTo('App\Models\Customers\Religious','religious_id','id');       
    }

    public function emergencyContact()
    {
        return $this->hasOne('App\Models\Customers\EmergencyContact','customer_id','id');
    }

    public function plan(){
       return $this->belongsTo('App\Models\MembershipPlan\Plan','plan_id','id');
    }
    /**
     * Get customer reports
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function flagged(){        
    	return $this->hasMany('App\Models\Reports\Report', 'node_id','id')->where('type','customer')->with('flagged_by');
    }

    public function getImageAttribute ($value) 
    {
        $directoryName = "customerProfileImages";
        return  $providerProfilePresignedUrl = presignedUrl($directoryName,$value);
        
    }

    public function setBirthDateAttribute ($value) 
    {
        $this->attributes['birth_date'] = dateFormat($value);
    }

    public function getBirthDateAttribute ($value) 
    {
        return showDateFormat($value);
    }

    /*public function setFirstNameAttribute($value)
    {
        $this->attributes['first_name'] = encryptString($value);
    }*/

    /*public function setLastNameAttribute($value)
    {
        $this->attributes['last_name'] = encryptString($value);
    }

    public function setEmailAttribute($value)
    {
        $this->attributes['email'] = encryptString($value);
    }

    public function setPhoneAttribute($value)
    {
        $this->attributes['phone'] = encryptString($value);
    }

    public function setAddressAttribute($value)
    {
        $this->attributes['address'] = encryptString($value);
    }*/

    /*public function getFirstNameAttribute($value)
    {
        return decryptString($value);
    }*/

    public function getActivitylogOptions(): LogOptions
    {
        return LogOptions::defaults()
        ->useLogName('Customer')
        ->logOnlyDirty()
        ->logOnly(['first_name','last_name','email','password','phone','address','birth_date','gender','race_id','ethnicity_id','religious_id','relationship_id','image','status','provider_preference','language_id','plan_id','purchased_date','register_status','register_step','email_confirmed','timezone','height_feet','height_inch','weight'])
        ->setDescriptionForEvent(fn(string $eventName) => "This customer has been {$eventName}");
    }

    // public function getCreatedAtAttribute ($value) 
    // { 
    //     // $timezone = !empty(request()->admin_time_zone) ? request()->admin_time_zone : "America/New_York";
    //     // $dateTime = Carbon::parse($value)->timezone($timezone);
    //     // return date('m/d/Y h:i A',strtotime($dateTime));
    //     return $value;
    // }

    public function getLastUpdatedAtAttribute ($value) 
    { 
        // if(!empty($value)){
        //     $timezone = !empty(request()->admin_time_zone) ? request()->admin_time_zone : "America/New_York";
        //     $dateTime = Carbon::parse($value)->timezone($timezone);
        //     return date('m/d/Y h:i A',strtotime($dateTime));
        // }
        return $value;    
    }

    public function getLastViewedAtAttribute ($value) 
    { 
        // if(!empty($value)){
        //     $timezone = !empty(request()->admin_time_zone) ? request()->admin_time_zone : "America/New_York";
        //     $dateTime = Carbon::parse($value)->timezone($timezone);
        //     return date('m/d/Y h:i A',strtotime($dateTime));
        // }
        return $value;
    }

    public function getCreatedAtAttribute ($created_at) 
   { 
    // return convertTimeUtcToLocal($created_at);
    return $created_at;
     
   }

   public function getUpdatedAtAttribute ($updated_at) 
   { 
    //   return convertTimeUtcToLocal($updated_at);
      return $updated_at;
   }

   public function setCreatedAtAttribute () 
   { 
    $this->attributes['created_at']=date('Y-m-d H:i:s');
    // $this->attributes['created_at']=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));

   }
   
   public function setUpdatedAtAttribute () 
   { 
      $this->attributes['updated_at']=date('Y-m-d H:i:s');
    //   $this->attributes['updated_at']=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));
   } 
}
