<?php

namespace App\Models\Providers;

use Illuminate\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Laravel\Lumen\Auth\Authorizable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Mail;
use Spatie\Activitylog\Traits\LogsActivity;
use Spatie\Activitylog\LogOptions;
use App\Jobs\SendTwoFactorJob;
use Carbon\Carbon;

class Provider extends Model implements AuthenticatableContract, AuthorizableContract
{
    use Authenticatable, Authorizable, HasFactory, Notifiable,LogsActivity;

    protected $table = 'providers';
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'first_name',
         'last_name', 
         'email',
         'birth_date',
         'gender',
         'other_gender',
         'password',
         'phone_code',
         'phone', 
         'address',
         'apartment',
         'swift_number',
         'image', 
         'experience',
         'bio',
         'register_status',
         'register_step',
         'email_confirmed',
         'email_confirm_token',
         'other_specialization',
         'other_language',
         'two_factor',
         'two_factor_code',
         'two_factor_expires_at',
         'otp',
         'otp_validity',
         'sms_otp',
         'rejection_reason',
         'rejection_date',
         'latitude',
         'longitude',
         'timezone',
         'next_available_slot',
         'pin',
         'login_attempt',
         'is_blocked',
         'login_attempt_time',
         'state',
         'zipcode',
         'token',
         'country',
         'city',
         'verified',
         'rating',
         'reviews',
         'license_update_status'
    ];

    /**
     * The attributes excluded from the model's JSON form.
     *
     * @var array
     */
    protected $hidden = [
        'password',
    ];

    // protected $casts = [
    //     'created_at' => 'datetime:Y-m-d H:i:s',
    //     'updated_at' => 'datetime:Y-m-d H:i:s',
    // ];

    //protected $dates = [ 'created_at', 'updated_at', ];

    public function generateTwoFactorCode()
    {
        $digits = 4;
        $toEmail = decryptString($this->email);
        //$toEmail = $this->email;
        $toName = decryptString($this->first_name);
        $otpCode = rand(pow(10, $digits-1), pow(10, $digits)-1);

        $sendMail =  new SendTwoFactorJob($toName,$toEmail,$otpCode); 
        dispatch($sendMail);

        $this->timestamps = false;
        $this->two_factor_code = $otpCode;
        $this->two_factor_expires_at = strtotime(date("Y-m-d H:i:s"));
        $this->save();  

    }

    public function resetTwoFactorCode()
    {
        $this->timestamps = false;
        $this->two_factor_code = null;
        $this->two_factor_expires_at = null;
        $this->save();
    }



    public function licenses()
    {
        return $this->hasMany('App\Models\Providers\ProviderLicense','provider_id','id');
    }
    
    public function licensePending()
    {
        return $this->hasMany('App\Models\Providers\ProviderLicensesPendingApprove','provider_id','id');
    }

    public function certifications()
    {
        return $this->hasMany('App\Models\Providers\ProviderCertification','provider_id','id');
    }

    public function fbiBackgrounds()
    {
        return $this->hasOne('App\Models\Providers\ProviderFbiBackground','provider_id','id');
    }

    public function childAbuseHistories()
    {
        return $this->hasOne('App\Models\Providers\ProviderChildAbuseHistory','provider_id','id');
    }

    public function liabilityInsurances()
    {
        return $this->hasOne('App\Models\Providers\ProviderLiabilityInsurance','provider_id','id');
    }

    public function professionalReferences()
    {
        return $this->hasMany('App\Models\Providers\ProviderProfessionalReference','provider_id','id');
    }


    /*public function specializations()
    {
        return $this->hasMany('App\Models\Providers\ProviderSpecialization','provider_id','id');
    }*/
    
    public function specializations()
    {
        return $this->belongsToMany( 'App\Models\Providers\Specialization', 'provider_specializations', 'provider_id', 'specialization_id' );
    } 
    public function languages()
    {
        return $this->belongsToMany( 'App\Models\Providers\Language', 'provider_languages', 'provider_id', 'language_id' );
    } 

    public function providerAvailability()
    {
        return $this->hasMany('App\Models\Providers\ProviderAvailability','provider_id','id');
    }   

    
    public function ProviderStripeData()
    {
        return $this->hasMany('App\Models\Providers\ProviderStripeData','provider_id','id');
    }   
    /**
     * Get customer role
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function oauth(){        
    	return $this->belongsTo('App\Models\Users\Oauth', 'id','user_fk')->where('role_fk','3');
    }

    public function setBirthDateAttribute ($value) 
    {
        $this->attributes['birth_date'] = dateFormat($value);
    }

    public function getBirthDateAttribute ($value) 
    {
        return showDateFormat($value);
    }

    public function getRejectionDateAttribute ($value) 
    {
        return rejectionDateTimeFormat($value);
    }

    public function getImageAttribute ($value) 
    {
        //dd(request()->time_zone);
        $directoryName = "providerProfileImages";
        return  $providerProfilePresignedUrl = presignedUrl($directoryName,$value);
        
    }

    /*public function getOrderCountAttribute () {
        $value = $this->getAttributeValue('status');
        return $value;
    }*/


    // Customize log description
    public function getActivitylogOptions(): LogOptions
    {
        return LogOptions::defaults()
        ->useLogName('Provider')
        ->logOnlyDirty()
        ->logOnly(['first_name', 'last_name','email','birth_date','gender','phone','address','apartment','swift_number','password','experience','bio','image','rating','reviews','application_status','status','register_status','register_step','email_confirmed','other_specialization','other_language','language','timezone','rejection_reason','rejection_date'])
        ->setDescriptionForEvent(fn(string $eventName) => "This provider has been {$eventName}");
    }

    // public function getCreatedAtAttribute ($value) 
    // { 
    //    // $date = Carbon::createFromFormat('Y-m-d H:i:s', $value, 'UTC')->setTimezone(config('constant.timezone'));
    //     // $timezone = !empty(request()->admin_time_zone) ? request()->admin_time_zone : config('constant.timezone');
    //     // $timezone = config('constant.timezone');
    //     // $dateTime = Carbon::parse($value)->timezone($timezone);
    //     // return date('m/d/Y h:i A',strtotime($dateTime));
    //     return date('m/d/Y H:i:s',strtotime($value));
    // }

    
}
